﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls.Primitives;
using System.Windows;
using System.Windows.Threading;
using System.Windows.Controls;
using System.Windows.Data;

namespace Sandbox
{
    /// <summary>
    /// Attached Behaviors for the TextBox class.
    /// </summary>
    public static class Behaviours
    {
        public static readonly DependencyProperty UpdateTargetWhenTextChangedProperty = DependencyProperty.RegisterAttached(
            "UpdateTargetWhenTextChanged",
            typeof(bool),
            typeof(TextBox),
            new FrameworkPropertyMetadata(OnUpdateTargetWhenTextChanged));

        [AttachedPropertyBrowsableForType(typeof(TextBoxBase))]
        public static bool GetUpdateTargetWhenTextChanged(TextBoxBase d)
        {
            return (bool)d.GetValue(UpdateTargetWhenTextChangedProperty);
        }

        public static void SetUpdateTargetWhenTextChanged(TextBoxBase d, bool value)
        {
            d.SetValue(UpdateTargetWhenTextChangedProperty, value);
        }

        static void OnUpdateTargetWhenTextChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var tb = (TextBoxBase)d;

            if ((bool)e.NewValue)
                tb.TextChanged += OnTextChanged;
            else
                tb.TextChanged -= OnTextChanged;
        }

        static void OnTextChanged(object sender, TextChangedEventArgs e)
        {
            var tb = (TextBoxBase)sender;
            tb.Dispatcher.BeginInvoke(DispatcherPriority.Input, new Action(delegate
            {
                MultiBindingExpression binding = BindingOperations.GetMultiBindingExpression(tb, TextBox.TextProperty);

                if (binding != null)
                {
                    binding.UpdateTarget();
                    binding.BindingExpressions[0].UpdateTarget();
                }
            }));
        }
    }
}
